@@ -1,12 +1,12 @@
 /*
  * checkconf - Read and repeat configuration file to output.
  *
- * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
  */
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -16,6 +16,7 @@
 #include "options.h"
 #include "util.h"
 #include "dname.h"
+#include "rrl.h"
 
 extern char *optarg;
 extern int optind;
@@ -26,6 +27,14 @@ extern int optind;
 		return; 			\
 	}
 
+#define ZONE_GET_OUTGOING(NAME, VAR)			\
+	if (strcasecmp(#NAME, (VAR)) == 0) {		\
+		acl_options_t* acl; 			\
+		for(acl=zone->NAME; acl; acl=acl->next)	\
+			quote(acl->ip_address_spec);	\
+		return; 				\
+	}
+
 #define ZONE_GET_STR(NAME, VAR) 		\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 	\
 		quote(zone->NAME); 		\
@@ -37,6 +46,12 @@ extern int optind;
 		printf("%s\n", zone->NAME?"yes":"no"); 	\
 	}
 
+#define ZONE_GET_RRL(NAME, VAR, PATTERN) 			\
+	if (strcasecmp(#NAME, (VAR)) == 0) { 		\
+		zone_print_rrl_whitelist("", PATTERN->NAME);	\
+		return;					\
+	}
+
 #define SERV_GET_BIN(NAME, VAR) 			\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 		\
 		printf("%s\n", opt->NAME?"yes":"no"); 	\
@@ -50,7 +65,7 @@ extern int optind;
 
 #define SERV_GET_INT(NAME, VAR) 		\
 	if (strcasecmp(#NAME, (VAR)) == 0) { 	\
-		printf("%d\n", opt->NAME); 	\
+		printf("%d\n", (int) opt->NAME); 	\
 		return; 			\
 	}
 
@@ -63,6 +78,21 @@ extern int optind;
 		return;						\
 	}
 
+#ifdef RATELIMIT
+static void zone_print_rrl_whitelist(const char* s, uint16_t w)
+{
+	int i;
+	if(w==rrl_type_all) {
+		printf("%sall\n", s);
+		return;
+	}
+	for(i=0x01; i <= 0x80; i<<=1) {
+		if( (w&i) )
+			printf("%s%s\n", s, rrltype2str(i));
+	}
+}
+#endif /* RATELIMIT */
+
 static char buf[BUFSIZ];
 
 static char *
@@ -77,7 +107,7 @@ underscore(const char *s) {
 			buf[i++] = *j;
 		}
 		j++;
-		if (i > BUFSIZ) {
+		if (i >= BUFSIZ) {
 			return NULL;
 		}
 	}
@@ -99,6 +129,7 @@ usage(void)
 	fprintf(stderr, "-h		Print this help information.\n");
 	fprintf(stderr, "-o option	Print value of the option specified to stdout.\n");
 	fprintf(stderr, "-z zonename	Print option value for the zone given.\n");
+	fprintf(stderr, "-a keyname	Print algorithm name for the TSIG key.\n");
 	fprintf(stderr, "-s keyname	Print base64 secret blob for the TSIG key.\n");
 	exit(1);
 }
@@ -182,9 +213,18 @@ print_acl(const char* varname, acl_options_t* acl)
 	}
 }
 
+static void
+print_acl_ips(const char* varname, acl_options_t* acl)
+{
+	while(acl)
+	{
+		printf("\t%s %s\n", varname, acl->ip_address_spec);
+		acl=acl->next;
+	}
+}
 
 void
-config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *z)
+config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o, const char *z)
 {
 	zone_options_t* zone;
 	ip_address_option_t* ip;
@@ -194,7 +234,11 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 		key_options_t* key = opt->keys;
 		for( ; key ; key=key->next) {
 			if(strcmp(key->name, k) == 0) {
-				quote(key->secret);
+				if (s) {
+					quote(key->secret);
+				} else {
+					quote(key->algorithm);
+				}
 				return;
 			}
 		}
@@ -223,8 +267,12 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 				ZONE_GET_ACL(provide_xfr, o);
 				ZONE_GET_ACL(allow_notify, o);
 				ZONE_GET_ACL(notify, o);
-				ZONE_GET_ACL(outgoing_interface, o);
+				ZONE_GET_BIN(notify_retry, o);
+				ZONE_GET_OUTGOING(outgoing_interface, o);
 				ZONE_GET_BIN(allow_axfr_fallback, o);
+#ifdef RATELIMIT
+				ZONE_GET_RRL(rrl_whitelist, o, zone);
+#endif
 				printf("Zone option not handled: %s %s\n", z, o);
 				exit(1);
 			}
@@ -242,6 +290,7 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 		/* str */
 		SERV_GET_STR(database, o);
 		SERV_GET_STR(identity, o);
+		SERV_GET_STR(nsid, o);
 		SERV_GET_STR(logfile, o);
 		SERV_GET_STR(pidfile, o);
 		SERV_GET_STR(chroot, o);
@@ -250,12 +299,24 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 		SERV_GET_STR(difffile, o);
 		SERV_GET_STR(xfrdfile, o);
 		SERV_GET_STR(port, o);
+#if defined(BIND8_STATS) && defined(USE_ZONE_STATS)
+		SERV_GET_STR(zonestatsfile, o);
+#endif
 		/* int */
 		SERV_GET_INT(server_count, o);
 		SERV_GET_INT(tcp_count, o);
+		SERV_GET_INT(tcp_query_count, o);
+		SERV_GET_INT(tcp_timeout, o);
+		SERV_GET_INT(ipv4_edns_size, o);
+		SERV_GET_INT(ipv6_edns_size, o);
 		SERV_GET_INT(statistics, o);
 		SERV_GET_INT(xfrd_reload_timeout, o);
 		SERV_GET_INT(verbosity, o);
+#ifdef RATELIMIT
+		SERV_GET_INT(rrl_size, o);
+		SERV_GET_INT(rrl_ratelimit, o);
+		SERV_GET_INT(rrl_whitelist_ratelimit, o);
+#endif
 
 		if(strcasecmp(o, "zones") == 0) {
 			RBTREE_FOR(zone, zone_options_t*, opt->zone_options)
@@ -282,12 +343,20 @@ config_test_print_server(nsd_options_t* opt)
 	printf("\thide-version: %s\n", opt->hide_version?"yes":"no");
 	print_string_var("database:", opt->database);
 	print_string_var("identity:", opt->identity);
+	print_string_var("nsid:", opt->nsid);
 	print_string_var("logfile:", opt->logfile);
 	printf("\tserver_count: %d\n", opt->server_count);
 	printf("\ttcp_count: %d\n", opt->tcp_count);
+	printf("\ttcp_query_count: %d\n", opt->tcp_query_count);
+	printf("\ttcp_timeout: %d\n", opt->tcp_timeout);
+	printf("\tipv4-edns-size: %d\n", (int) opt->ipv4_edns_size);
+	printf("\tipv6-edns-size: %d\n", (int) opt->ipv6_edns_size);
 	print_string_var("pidfile:", opt->pidfile);
 	print_string_var("port:", opt->port);
 	printf("\tstatistics: %d\n", opt->statistics);
+#if defined(BIND8_STATS) && defined(USE_ZONE_STATS)
+	printf("\tzone-stats-file: %s\n", opt->zonestatsfile);
+#endif
 	print_string_var("chroot:", opt->chroot);
 	print_string_var("username:", opt->username);
 	print_string_var("zonesdir:", opt->zonesdir);
@@ -295,6 +364,11 @@ config_test_print_server(nsd_options_t* opt)
 	print_string_var("xfrdfile:", opt->xfrdfile);
 	printf("\txfrd_reload_timeout: %d\n", opt->xfrd_reload_timeout);
 	printf("\tverbosity: %d\n", opt->verbosity);
+#ifdef RATELIMIT
+	printf("\trrl-size: %d\n", (int)opt->rrl_size);
+	printf("\trrl-ratelimit: %d\n", (int)opt->rrl_ratelimit);
+	printf("\trrl-whitelist-ratelimit: %d\n", (int)opt->rrl_whitelist_ratelimit);
+#endif
 
 	for(ip = opt->ip_addresses; ip; ip=ip->next)
 	{
@@ -312,11 +386,15 @@ config_test_print_server(nsd_options_t* opt)
 		printf("\nzone:\n");
 		print_string_var("name:", zone->name);
 		print_string_var("zonefile:", zone->zonefile);
+#ifdef RATELIMIT
+		zone_print_rrl_whitelist("\trrl-whitelist: ", zone->rrl_whitelist);
+#endif
 		print_acl("allow-notify:", zone->allow_notify);
 		print_acl("request-xfr:", zone->request_xfr);
+		printf("\tnotify-retry: %d\n", zone->notify_retry);
 		print_acl("notify:", zone->notify);
 		print_acl("provide-xfr:", zone->provide_xfr);
-		print_acl("outgoing-interface:", zone->outgoing_interface);
+		print_acl_ips("outgoing-interface:", zone->outgoing_interface);
 		printf("\tallow-axfr-fallback: %s\n", zone->allow_axfr_fallback?"yes":"no");
 	}
 
@@ -369,7 +447,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 			fprintf(stderr, "%s: cannot base64 decode tsig secret: for key %s.\n", filename, key->name);
 			errors ++;
 		}
-		if(tsig_get_algorithm_by_name(key->algorithm) != NULL)
+		if(tsig_get_algorithm_by_name(key->algorithm) == NULL)
 		{
 			fprintf(stderr, "%s: bad tsig algorithm %s: for key \
 %s.\n", filename, key->algorithm, key->name);
@@ -384,7 +462,16 @@ additional_checks(nsd_options_t* opt, const char* filename)
 			filename, opt->statistics);
 		errors ++;
 	}
+#  ifndef USE_ZONE_STATS
+	if(opt->zonestatsfile)
+	{
+		fprintf(stderr, "%s: 'zone-stats-file: %s' but per zone BIND 8 statistics feature not enabled.\n",
+			filename, opt->zonestatsfile);
+		errors ++;
+	}
+#  endif
 #endif
+
 #ifndef HAVE_CHROOT
 	if(opt->chroot != 0)
 	{
@@ -444,14 +531,18 @@ main(int argc, char* argv[])
 {
 	int c;
 	int verbose = 0;
+	int key_sec = 0;
 	const char * conf_opt = NULL; /* what option do you want? Can be NULL -> print all */
 	const char * conf_zone = NULL; /* what zone are we talking about */
 	const char * conf_key = NULL; /* what key is needed */
 	const char* configfile;
 	nsd_options_t *options;
 
+	log_init("nsd-checkconf");
+
+
         /* Parse the command line... */
-        while ((c = getopt(argc, argv, "vo:s:z:")) != -1) {
+        while ((c = getopt(argc, argv, "vo:a:s:z:")) != -1) {
 		switch (c) {
 		case 'v':
 			verbose = 1;
@@ -459,9 +550,21 @@ main(int argc, char* argv[])
 		case 'o':
 			conf_opt = optarg;
 			break;
-		case 's':
+		case 'a':
+			if (conf_key) {
+				fprintf(stderr, "Error: cannot combine -a with -s or other -a.\n");
+				exit(1);
+			}
 			conf_key = optarg;
 			break;
+		case 's':
+			if (conf_key) {
+				fprintf(stderr, "Error: cannot combine -s with -a or other -s.\n");
+				exit(1);
+			}
+			conf_key = optarg;
+			key_sec = 1;
+			break;
 		case 'z':
 			conf_zone = optarg;
 			break;
@@ -478,12 +581,13 @@ main(int argc, char* argv[])
 
 	/* read config file */
 	options = nsd_options_create(region_create(xalloc, free));
+	tsig_init(options->region);
 	if (!parse_options_file(options, configfile) ||
 	   !additional_checks(options, configfile)) {
 		exit(2);
 	}
 	if (conf_opt || conf_key) {
-		config_print_zone(options, conf_key, underscore(conf_opt), conf_zone);
+		config_print_zone(options, conf_key, key_sec, underscore(conf_opt), conf_zone);
 	} else {
 		if (verbose) {
 			printf("# Read file %s: %d zones, %d keys.\n",
